<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2022 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_).'addifyrequestaquote/classes/AddifyrequestaquoteClass.php';
include_once(_PS_MODULE_DIR_).'addifyrequestaquote/classes/AddifyrequestsallClass.php';
include_once(_PS_MODULE_DIR_).'addifyrequestaquote/classes/AddifycartsallClass.php';

class Addifyrequestaquote extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'addifyrequestaquote';
        $this->tab = 'front_office_features';
        $this->version = '1.3.0';
        $this->author = 'Addify';
        $this->need_instance = 0;
        $this->module_key = 'eb37fca633aaca3aa559dcf33e62579f';
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Addify Request A Quote');
        $this->description = $this->l('Request a quote for B2B customer');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_LIVE_MODE', 1);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_OFFERED_MODE', 1);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_SEMAIL', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_ACTIVE', 0);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_LABLE', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_PLACEHOLDER', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_FILEUPLOAD_ACTIVE', 0);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_FILEUPLOAD_LABLE', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_ACTIVE', 0);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_LABLE', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_OPTIONS', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_ACTIVE', 0);
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_LABLE', '');
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_OPTIONS', '');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') && 
            $this->registerHook('displayTop') &&
            $this->registerHook('displayProductPriceBlock') &&
            $this->registerHook('displayCustomerAccount') &&
            $this->registerHook('registerGDPRConsent') && 
            $this->registerHook('actionDeleteGDPRCustomer') && 
            $this->registerHook('actionExportGDPRData') && 
            $this->registerHook('displayGDPRConsent');
    }

    public function uninstall()
    {
        include(dirname(__FILE__).'/sql/uninstall.php');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_LIVE_MODE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_OFFERED_MODE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_SEMAIL');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_TEXTTYPE_ACTIVE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_TEXTTYPE_LABLE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_TEXTTYPE_PLACEHOLDER');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_FILEUPLOAD_ACTIVE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_FILEUPLOAD_LABLE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_MULTICHECK_ACTIVE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_MULTICHECK_LABLE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_MULTICHECK_OPTIONS');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_DROPDOWN_ACTIVE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_DROPDOWN_LABLE');
        Configuration::deleteByName('ADDIFYREQUESTAQUOTE_DROPDOWN_OPTIONS');

        return parent::uninstall() &&
            $this->unregisterHook('header') &&
            $this->unregisterHook('displayBackOfficeHeader') && 
            $this->unregisterHook('displayTop') &&
            $this->unregisterHook('displayProductPriceBlock') &&
            $this->unregisterHook('displayCustomerAccount') && 
            $this->unregisterHook('registerGDPRConsent') && 
            $this->unregisterHook('actionDeleteGDPRCustomer') && 
            $this->unregisterHook('actionExportGDPRData') && 
            $this->unregisterHook('displayGDPRConsent');
    }


    /**
     * Load the configuration form
     */
    public function getContent()
    {
        if(Tools::isSubmit('Submitrequestsall')) {
            $id_addifyrequestaquote = Tools::getValue('id_addifyrequestaquote'); 
            $requestaquote = new AddifyrequestsallClass((int)$id_addifyrequestaquote);
            $requestaquote->status = Tools::getValue('status');
            $requestaquote->id_product = Tools::getValue('allpids');
            $requestaquote->id_product_attribute = Tools::getValue('allpattr');
            $requestaquote->admin_price = Tools::getValue('admin_price');
            $requestaquote->product_info = Tools::getValue('allpinfo');
            $requestaquote->product_image = Tools::getValue('allpimages');
            $requestaquote->product_name = Tools::getValue('allpname');
            $requestaquote->product_sku = Tools::getValue('allpsku');
            $requestaquote->offered_price = Tools::getValue('offered_price');
            $requestaquote->quantity = Tools::getValue('quantity');
            $requestaquote->name = Tools::getValue('name');
            $requestaquote->email = Tools::getValue('email');
            $requestaquote->description = Tools::getValue('description');
            $requestaquote->cartlink = Tools::getValue('cartlink');

            if(Tools::getValue('addifyraq_enable')==1){
            $requestaquote->sendlinkflag = 1;
            if(Tools::getValue('cartlink') != '')
            $this->sendMail(Tools::getValue('email'),Tools::getValue('name'),Tools::getValue('cartlink'));
            }
            $requestaquote->adminnote = Tools::getValue('adminnote');
            
            $allitems=explode(',',Tools::getValue('allpids'));
            $itemscount=count($allitems);
            for($i=0;$i<$itemscount;$i++) {
                if($i==0) {
                    $allitems=Tools::getValue('adminprice'.$i);
                } else {
                    $allitems=$allitems.','.Tools::getValue('adminprice'.$i);
                }     
            }

            $requestaquote->save();
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
        }


        if(Tools::isSubmit('updateaddifyrequestsall')) {

            $id_addifyrequestaquote = Tools::getValue('id_addifyrequestaquote'); 
            $requestaquote = new AddifyrequestsallClass((int)$id_addifyrequestaquote);
            $status = $requestaquote->status;
            $allid_product = $requestaquote->id_product;
            $allid_productarr = explode(',', $requestaquote->id_product);
            $allpattr = $requestaquote->id_product_attribute;
            $allpattrarr = explode(',',$requestaquote->id_product_attribute);
            $allpinfo = $requestaquote->product_info;
            $allpinfoarr = explode(',',$requestaquote->product_info);
            $allpimages = $requestaquote->product_image;
            $allpimagesarr = explode(',',$requestaquote->product_image);
            $allpname = $requestaquote->product_name;
            $allpnamearr = explode(',',$requestaquote->product_name);
            $allpsku = $requestaquote->product_sku;
            $allpskuarr = explode(',',$requestaquote->product_sku);
            $offered_price = $requestaquote->offered_price ;
            $offered_pricearr = explode(',',$requestaquote->offered_price) ;
            $original_price = $requestaquote->original_price ;
            $original_pricearr = explode(',',$requestaquote->original_price) ;
            $quantity = $requestaquote->quantity ;
            $quantityarr = explode(',',$requestaquote->quantity) ;
            $name = $requestaquote->name ;
            $email = $requestaquote->email ;
            $description = $requestaquote->description ;
            $adminnote = $requestaquote->adminnote;
            $file = $requestaquote->file ;
            $dropdown = $requestaquote->dropdown ;
            $texttype = $requestaquote->texttype ;
            $multiselect = $requestaquote->multiselect ;
            $admin_price = $requestaquote->admin_price ;
            $sendlinkflag = $requestaquote->sendlinkflag;
            $cartlink = $requestaquote->cartlink;
            $cartexpire = $requestaquote->cartexpire;
            if ($cartexpire != ''){
                $cartexpire = " ---- Expiry Date : "  . $cartexpire;
            }
            $cartkey = $requestaquote->cartkey;
            $admin_pricearr = explode(',',$requestaquote->admin_price) ;
            $number_of_products = count($allid_productarr);

            $addifyraq_ajax_link = $this->context->link->getAdminLink('AdminAddifyrequestaquoteCartlink',true);
            $backurl=$this->context->link->getAdminLink('AdminModules', true) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

            $this->context->smarty->assign(array(
            'id_addifyrequestaquote' => $id_addifyrequestaquote,
            'status' => $status,
            'allid_product' => $allid_product,
            'allid_productarr' => $allid_productarr,
            'allpattr' => $allpattr,
            'allpattrarr' => $allpattrarr,
            'allpinfo' => $allpinfo,
            'allpinfoarr' => $allpinfoarr,
            'allpimages' => $allpimages,
            'allpimagesarr' => $allpimagesarr,
            'allpname' => $allpname,
            'allpnamearr' => $allpnamearr,
            'allpsku' => $allpsku,
            'allpskuarr' => $allpskuarr,
            'offered_price' => $offered_price,
            'offered_pricearr' => $offered_pricearr,
            'original_price' => $original_price,
            'original_pricearr' => $original_pricearr,
            'quantity' => $quantity,
            'quantityarr' => $quantityarr,
            'admin_price' => $admin_price,
            'admin_pricearr' => $admin_pricearr,
            'name' => $name,
            'email' => $email,
            'description' => $description,
            'adminnote' => $adminnote,
            'file' => $file ,
            'dropdown' => $dropdown ,
            'texttype'=> $texttype ,
            'multiselect'=> $multiselect ,
            'sendlinkflag' => $sendlinkflag,
            'addifyraq_cartlink' => $cartlink,
            'cartexpire' => $cartexpire,
            'cartkey' => $cartkey,
            'curr' =>  Currency::getDefaultCurrency()->sign,
            'target_dir' => _PS_MODULE_DIR_.'addifyrequestaquote/views/img/',
            'baseUrl' => $this->context->shop->getBaseURL(true),
            'number_of_products' => $number_of_products,
            'backurl' => $backurl,
            'aadifyraq_offered_mode' => Configuration::get('ADDIFYREQUESTAQUOTE_OFFERED_MODE'),
            'addifyraq_ajax_link' => $addifyraq_ajax_link
            ));
            $this->context->smarty->assign('module_dir', $this->_path);
            $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/allrequest.tpl');
            return $output;
        }

        /**
         * If values have been submitted in the form, process.
         */
        if(Tools::isSubmit('Submitconfig')) {
              $this->processconfigSave();
        }

        $this->postProcess();

        if (Tools::isSubmit('add'.$this->name) || Tools::isSubmit('update'.$this->name)) {
                return $this->renderForm();
        }
        if(Tools::isSubmit('Submitconfig')) {
              $this->processconfigSave();
        }

        $this->context->smarty->assign(array(
            'aadifyraq_live_mode' => Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE'),
            'aadifyraq_offered_mode' => Configuration::get('ADDIFYREQUESTAQUOTE_OFFERED_MODE'),
            'aadifyraq_semail' => Configuration::get('ADDIFYREQUESTAQUOTE_SEMAIL'),
            'aadifyraq_texttype_active' => Configuration::get('ADDIFYREQUESTAQUOTE_TEXTTYPE_ACTIVE'),
            'aadifyraq_texttype_lable' => Configuration::get('ADDIFYREQUESTAQUOTE_TEXTTYPE_LABLE'),
            'aadifyraq_texttype_placeholder'=>Configuration::get('ADDIFYREQUESTAQUOTE_TEXTTYPE_PLACEHOLDER'),
            'aadifyraq_fileupload_active' => Configuration::get('ADDIFYREQUESTAQUOTE_FILEUPLOAD_ACTIVE'),
            'aadifyraq_fileupload_lable' => Configuration::get('ADDIFYREQUESTAQUOTE_FILEUPLOAD_LABLE'),
            'aadifyraq_multicheck_active' => Configuration::get('ADDIFYREQUESTAQUOTE_MULTICHECK_ACTIVE'),
            'aadifyraq_multicheck_lable' => Configuration::get('ADDIFYREQUESTAQUOTE_MULTICHECK_LABLE'),
            'aadifyraq_multicheck_options' => Configuration::get('ADDIFYREQUESTAQUOTE_MULTICHECK_OPTIONS'),
            'aadifyraq_dropdown_active' => Configuration::get('ADDIFYREQUESTAQUOTE_DROPDOWN_ACTIVE'),
            'aadifyraq_dropdown_labe' => Configuration::get('ADDIFYREQUESTAQUOTE_DROPDOWN_LABLE'),
            'aadifyraq_dropdown_options' => Configuration::get('ADDIFYREQUESTAQUOTE_DROPDOWN_OPTIONS'),
        ));

        $this->context->smarty->assign('module_dir', $this->_path);

        $output = $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderList().$this->renderRequestList();
    }

    protected function processconfigSave()
    {   
        if (Tools::getValue('aadifyraq_semail') && !validate::isEmail(Tools::getValue('aadifyraq_semail'))) {
            $this->context->controller->errors[] = $this->l('Shop/Manager Email is not valid .');
        }

        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }

        Configuration::updateValue('ADDIFYREQUESTAQUOTE_LIVE_MODE', Tools::getValue('addifyraq_enable'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_OFFERED_MODE', Tools::getValue('addifyraq_offered'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_SEMAIL', Tools::getValue('aadifyraq_semail'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_ACTIVE', Tools::getValue('aadifyraq_texttype_active'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_LABLE', Tools::getValue('aadifyraq_texttype_lable'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_TEXTTYPE_PLACEHOLDER', Tools::getValue('aadifyraq_texttype_placeholder'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_FILEUPLOAD_ACTIVE', Tools::getValue('aadifyraq_fileupload_active'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_FILEUPLOAD_LABLE', Tools::getValue('aadifyraq_fileupload_lable'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_ACTIVE', Tools::getValue('aadifyraq_multicheck_active'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_LABLE', Tools::getValue('aadifyraq_multicheck_lable'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_MULTICHECK_OPTIONS', Tools::getValue('aadifyraq_multicheck_options'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_ACTIVE', Tools::getValue('aadifyraq_dropdown_active'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_LABLE', Tools::getValue('aadifyraq_dropdown_labe'));
        Configuration::updateValue('ADDIFYREQUESTAQUOTE_DROPDOWN_OPTIONS', Tools::getValue('aadifyraq_dropdown_options'));

        Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=6&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
    }

    protected function renderList()
    {
        $fields_list = array(
            'id_addifyrequestaquote' => array(
                'title' => $this->l('ID'),
                'width' => 120,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'title' => array(
                'title' => $this->l('Title'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'priority' => array(
                'title' => $this->l('Priority'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'active' => array(
                'title' => $this->l('Status'),
                'width' => 140,
                'active' => 'status',
                'type' => 'bool',
                'search' => false,
                'orderby' => false
            ),
            'date_add' => array(
                'title' => $this->l('Added Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
            'date_upd' => array(
                'title' => $this->l('Update Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
        );

        // multishop feature
        if (Shop::isFeatureActive()) {
            $this->fields_list['id_shop'] = array(
                'title' => $this->l('ID Shop'),
                'align' => 'center',
                'width' => 25,
                'type' => 'int'
            );
        }

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->identifier = 'id_addifyrequestaquote';
        $helper->actions = array('edit', 'delete');
        $helper->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );
        $helper->listTotal = count(AddifyrequestaquoteClass::getListContent());
        $helper->show_toolbar = true;
        $helper->toolbar_btn['new'] =  array(
            'href' => AdminController::$currentIndex.'&configure='.$this->name.'&add'.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
            'desc' => $this->l('Add new')
        );

        $helper->title = $this->displayName.' Rules';
        $helper->table = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        return $helper->generateList(AddifyrequestaquoteClass::getListContent(), $fields_list);
    }

    protected function renderRequestList()
    {
        $fields_list = array(
            'id_addifyrequestaquote' => array(
                'title' => $this->l('ID'),
                'width' => 120,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'status' => array(
                'title' => $this->l('Status'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'name' => array(
                'title' => $this->l('Name'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'adminnote' => array(
                'title' => $this->l('Admin Note'),
                'width' => 140,
                'type' => 'text',
                'search' => false,
                'orderby' => false
            ),
            'date_add' => array(
                'title' => $this->l('Added Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
            'date_upd' => array(
                'title' => $this->l('Update Date'),
                'width' => 140,
                'type' => 'datetime',
                'search' => false,
                'orderby' => false
            ),
        );

        // multishop feature
        if (Shop::isFeatureActive()) {
            $this->fields_list['id_shop'] = array(
                'title' => $this->l('ID Shop'),
                'align' => 'center',
                'width' => 25,
                'type' => 'int'
            );
        }

        $helper = new HelperList();
        $helper->shopLinkType = '';
        $helper->simple_header = false;
        $helper->identifier = 'id_addifyrequestaquote';
        $helper->actions = array('edit', 'delete');
        $helper->bulk_actions = array(
            'delete' => array(
                'text' => $this->l('Delete selected'),
                'icon' => 'icon-trash',
                'confirm' => $this->l('Delete selected items?')
            )
        );
        $helper->listTotal = count(AddifyrequestsallClass::getListContent());
        $helper->show_toolbar = true;
        $helper->title = 'All Quotes';
        $helper->table = 'addifyrequestsall';
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        return $helper->generateList(AddifyrequestsallClass::getListContent(), $fields_list);
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->show_cancel_button = false;
        $helper->table = 'addifyrequestaquote';
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = 'id_'.$this->table;
        $helper->submit_action = 'submitAddifyrequestaquote';
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->back_url = $helper->currentIndex.'&token='.$helper->token;
        $products = array();

        if (Tools::getValue('id_addifyrequestaquote')) {
            $hidepriceBlock = new AddifyrequestaquoteClass((int)Tools::getValue('id_addifyrequestaquote'));
            $post_accessories_tab = explode(',', $hidepriceBlock->product);
            foreach ($post_accessories_tab as $id_product) {
                if (!$this->haveThisProduct($id_product, $products) &&
                    $product = AddifyrequestaquoteClass::getProductById($id_product)) {
                    $products[] = $product;
                }
            }
        } else {
            $products = array();
        }
        if ($post_accessories = Tools::getValue('inputRestrictedProducts')) {
            $post_accessories_tab = explode('-', $post_accessories);
            foreach ($post_accessories_tab as $id_product) {
                if (!$this->haveThisProduct($id_product, $products) &&
                    $product = AddifyrequestaquoteClass::getProductById($id_product)) {
                    $products[] = $product;
                }
            }
        }

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'products' => $products,
            'context_link' => Context::getContext()->link,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }
    public function haveThisProduct($id_product, $products)
    {
        foreach ($products as $product) {
            if ((int)$product['id_product'] == (int)$id_product) {
                return true;
            }
        }
        return false;
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $access_type = array(
            array('id' => '1', 'name' => $this->l('All Product')),
            array('id' => '2', 'name' => $this->l('Selected Product')),
        );

        if (Tools::getValue('id_addifyrequestaquote')) {
            $hidepriceBlock = new AddifyrequestaquoteClass((int)Tools::getValue('id_addifyrequestaquote'));
            $selected_categories = explode(',', $hidepriceBlock->categories);
        } else {
            $selected_categories = array();
        }

        if (Shop::isFeatureActive()) {
            $this->fields_form['input'][] = array(
                'type' => 'shop',
                'label' => $this->l('Shop association'),
                'name' => 'checkBoxShopAsso',
            );
        }

        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Create Rule'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'hidden',
                        'name' => 'id_addifyrequestaquote'
                    ),
                    array(
                        'type'          => 'text',
                        'label'         => $this->l('Title'),
                        'name'          => 'title',
                    ),
                    array(
                        'type'          => 'text',
                        'label'         => $this->l('Priority'),
                        'name'          => 'priority',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Active'),
                        'name' => 'active',
                        'is_bool' => true,
                        'desc' => $this->l('Enable/Disable this request a quote rule.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Hide Price '),
                        'name' => 'hideprice',
                        'desc' => $this->l('Enable/Disable hide price.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Hide Add To Cart Button'),
                        'desc' => $this->l('Enable/Disable hide add to cart button.'),
                        'name' => 'hidecart',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'          => 'color',
                        'label'         => $this->l('Button Text Color :'),
                        'name'          => 'text_color',
                    ),
                    array(
                        'type'          => 'color',
                        'label'         => $this->l('Button Background Color :'),
                        'name'          => 'text_bg_color',
                    ),
                    array(
                        'type' => 'group',
                        'label' => $this->l('Group Request A Quote'),
                        'name' => 'groupBox',
                        'values' => Group::getGroups(Context::getContext()->language->id),
                        'info_introduction' => $this->l('You now have three default customer groups.'),
                        'desc' => $this->l('Mark any/all of the customer group(s) which you would like to show Request A Quote.'),
                    ),
                    array(
                        'type' => 'select',
                        'label' => $this->l('Product type'),
                        'name' => 'type',
                        'options' => array(
                            'query' => $access_type,
                            'id' => 'id',
                            'name' => 'name'
                        ),
                        'col' => 2,
                    ),
                    array(
                        'type' => 'addifydisclaimer_product',
                        'label' => $this->l('Search Product'),
                        'name' => 'product',
                        'col' => 6,
                        'prefix' => '<i class="icon-search"></i>',
                        'hint' => $this->l('Request a quote button will shown for selected product.')
                    ),
                    array(
                        'type'  => 'categories',
                        'label' => $this->l('Categories'),
                        'name'  => 'category',
                        'col' => 6,
                        'tree'  => array(
                            'id' => 'type_category',
                            'use_checkbox' => true,
                            'disabled_categories' => null,
                            'selected_categories' => $selected_categories,
                            'root_category' => Context::getContext()->shop->getCategory()
                        )
                    ),

                    array(
                        'type' => 'text',
                        'label' => $this->l('Button Text'),
                        'name' => 'message',
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
                'buttons' => array(
                    'cancel' => array(
                        'title' => $this->l('Cancel'),
                        'class' => 'btn btn-default',
                        'icon' => 'process-icon-cancel',
                        'href' => AdminController::$currentIndex.'&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules'),
                    ),
                )
            ),
        );
    }

   
    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $fields_value = array();
        $accessed_groups_ids = array();

        if (Tools::getValue('id_addifyrequestaquote') && (Tools::isSubmit('submitAddifyrequestaquote') === false)) {
            $hidepriceBlock = new AddifyrequestaquoteClass((int)Tools::getValue('id_addifyrequestaquote'));
            $fields_value['id_addifyrequestaquote'] = $hidepriceBlock->id;
            $fields_value['title'] = $hidepriceBlock->title;
            $fields_value['active'] = $hidepriceBlock->active;
            $fields_value['groupBox'] = explode(',', $hidepriceBlock->groups);
            $fields_value['type'] = $hidepriceBlock->product_type;
            $fields_value['product'] = explode(',', $hidepriceBlock->product);
            $fields_value['priority'] = $hidepriceBlock->priority;
            $fields_value['category'] = explode(',', $hidepriceBlock->categories);
            $accessed_groups_ids= $fields_value['groupBox'];
            $fields_value['hideprice'] = $hidepriceBlock->hideprice;
            $fields_value['hideofferedprice'] = $hidepriceBlock->hideofferedprice;
            $fields_value['hidecart'] = $hidepriceBlock->hidecart;
            $fields_value['text_color'] = $hidepriceBlock->text_color;
            $fields_value['text_bg_color'] = $hidepriceBlock->text_bg_color;
            $fields_value['message'] = $hidepriceBlock->message;
        } else {
            $fields_value['id_addifyrequestaquote'] = Tools::getValue('id_addifyrequestaquote');
            $fields_value['active'] = (int)Tools::getValue('active');
            $fields_value['priority'] = Tools::getValue('priority');
            $fields_value['title'] = Tools::getValue('title');
            $fields_value['hideprice'] = Tools::getValue('hideprice');
            $fields_value['hideofferedprice'] = Tools::getValue('hideofferedprice');
            $fields_value['hidecart'] = Tools::getValue('hidecart');
            $fields_value['text_color'] = Tools::getValue('text_color');
            $fields_value['text_bg_color'] = Tools::getValue('text_bg_color');
            $fields_value['message'] = Tools::getValue('message');
            if (!Tools::getValue('groupBox')) {
                $fields_value['groupBox'] =  Tools::getValue('groupBox');
            } else {
                $fields_value['groupBox'] = implode(',', Tools::getValue('groupBox'));
            }
            $fields_value['type'] = Tools::getValue('type');
            if ($fields_value['type']==1) {
                $fields_value['product'] = '';
            } else {
                if (!Tools::getValue('product')) {
                    $fields_value['product'] = Tools::getValue('product');
                } else {
                    $fields_value['product'] = implode(',', Tools::getValue('product'));
                }
            }
            if ($fields_value['type']==2) {
                $fields_value['category'] = '';
            }else{
                if (Tools::getValue('category')) {
                    $fields_value['category'] = implode(',', Tools::getValue('category'));
                } else {
                    $fields_value['category'] ='' ;
                }
            }

            $accessed_groups_ids=  Tools::getValue('groupBox');
        }

        if (!is_array($accessed_groups_ids)) {
            $accessed_groups_ids = array();
        }
        // Added values of object Group
        $groups = Group::getGroups($this->context->language->id);
        if (empty($accessed_groups_ids)) {
            $preselected = array(Configuration::get('PS_UNIDENTIFIED_GROUP'), Configuration::get('PS_GUEST_GROUP'), Configuration::get('PS_CUSTOMER_GROUP'));
            $accessed_groups_ids = array_merge($accessed_groups_ids, $preselected);
        }
        foreach ($groups as $group) {
            $fields_value['groupBox_'.$group['id_group']] = (isset($accessed_groups_ids) && $accessed_groups_ids)? Tools::getValue('groupBox_'.$group['id_group'], (in_array($group['id_group'], $accessed_groups_ids))) : true;
        }

        return $fields_value;
    }
    /**
     * Save form data.
     */
    protected function processSave()
    {

        $call_back = 'add';
        $form_values = $this->getConfigFormValues();

        if (!isset($form_values) && !$form_values) {
            return $this->context->controller->errors[] = $this->l('Empty post values.');
        }
        if (!is_numeric($form_values['priority'])) {
            $this->context->controller->errors[] = $this->l('Priority value is not valid .');
        }


        if (count($this->context->controller->errors)) {
            return $this->context->controller->errors;
        }

        if (($id_addifyextendedhideprice = Tools::getValue('id_addifyrequestaquote'))) {
            $call_back = 'update';
            $hideprice = new AddifyrequestaquoteClass((int)$id_addifyextendedhideprice);
        } else {
            $hideprice = new AddifyrequestaquoteClass();
        }
        $hideprice->title = $form_values['title'];
        $hideprice->active = $form_values['active'];
        $hideprice->groups = $form_values['groupBox'];
        $hideprice->product_type = $form_values['type'];
        $hideprice->product = $form_values['product'];
        $hideprice->categories = $form_values['category'];       
        $hideprice->priority = $form_values['priority'];
        $hideprice->hideprice = $form_values['hideprice'];
        $hideprice->hideofferedprice = $form_values['hideofferedprice']; 
        $hideprice->hidecart = $form_values['hidecart'];
        if($form_values['text_color']==null)
            $form_values['text_color']='#4285f4';
        $hideprice->text_color = $form_values['text_color'];
        if($form_values['text_bg_color']==null)
            $form_values['text_bg_color']='#515356';
        $hideprice->text_bg_color = $form_values['text_bg_color'];
        if($form_values['message']==null)
            $form_values['message']='Add to Quote';
        $hideprice->message = $form_values['message'];

        if (!call_user_func(array($hideprice, $call_back))) {
            $this->context->controller->errors = sprintf($this->l('Something went wrong while performing operation %s'), $call_back);
        } else {
            // clear cache after add/update
            $this->flushCache();
            if($call_back=='add') {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=3&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
            }
        }
    }
    public function flushCache()
    {
        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();
        if (true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') &&
            is_callable(array('Tools', 'clearAllCache'))) {
            Tools::clearAllCache();
        }
    }
    /**
     * Save form data.
     */
    protected function postProcess()
    {

        if (Tools::isSubmit('submitAddifyrequestaquote')) {
            $this->processSave();
        } elseif (Tools::isSubmit('status'.$this->name)) {
            if (!AddifyrequestaquoteClass::setStatus('active', (int)Tools::getValue('id_addifyrequestaquote'))) {
                $this->context->controller->errors[] = $this->l('Status update failed.');
            } else {
                $this->context->controller->confirmations[] = $this->l('Status updated successfully.');
            }
        } elseif (Tools::isSubmit('delete'.$this->name)) {
            if (!Validate::isLoadedObject($hideprice = new AddifyrequestaquoteClass((int)Tools::getValue('id_addifyrequestaquote')))) {
                $this->context->controller->errors[] = $this->l('Object not found.');
            } else {
                if (!$hideprice->delete()) {
                    $this->context->controller->errors[] = $this->l('Rule cannot be deleted.');
                } else {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        } elseif (Tools::isSubmit('submitBulkdelete'.$this->name)) {
            $addifyextendedhidepriceBox = Tools::getValue('addifyrequestaquoteBox');
            if (isset($addifyextendedhidepriceBox) && $addifyextendedhidepriceBox) {
                $deleted = 0;
                foreach ($addifyextendedhidepriceBox as $id_addifyextendedhideprice) {
                    if (!Validate::isLoadedObject($hideprice = new AddifyrequestaquoteClass((int)$id_addifyextendedhideprice))) {
                        $this->context->controller->errors[] = $this->l('Object not found.');
                        break;
                    } else {
                        if ($hideprice->delete()) {
                            $deleted++;
                        }
                    }
                }
                if (count($this->context->controller->errors)) {
                    $this->context->controller->errors;
                } else {
                    $this->context->controller->confirmations[] = sprintf($this->l('%s records deleted successfully.'), $deleted);
                }
            }
        } elseif (Tools::isSubmit('deleteaddifyrequestsall')) {
            if (!Validate::isLoadedObject($hideprice = new AddifyrequestsallClass((int)Tools::getValue('id_addifyrequestaquote')))) {
                $this->context->controller->errors[] = $this->l('Object not found.');
            } else {
                if (!$hideprice->delete()) {
                    $this->context->controller->errors[] = $this->l('Rule cannot be deleted.');
                } else {
                    $this->context->controller->confirmations[] = $this->l('Rule deleted successfully.');
                }
            }
        } elseif (Tools::isSubmit('submitBulkdeleteaddifyrequestsall')) {
            $addifyextendedhidepriceBox = Tools::getValue('addifyrequestsallBox');
            if (isset($addifyextendedhidepriceBox) && $addifyextendedhidepriceBox) {
                $deleted = 0;
                foreach ($addifyextendedhidepriceBox as $id_addifyextendedhideprice) {
                    if (!Validate::isLoadedObject($hideprice = new AddifyrequestsallClass((int)$id_addifyextendedhideprice))) {
                        $this->context->controller->errors[] = $this->l('Object not found.');
                        break;
                    } else {
                        if ($hideprice->delete()) {
                            $deleted++;
                        }
                    }
                }
                if (count($this->context->controller->errors)) {
                    $this->context->controller->errors;
                } else {
                    $this->context->controller->confirmations[] = sprintf($this->l('%s records deleted successfully.'), $deleted);
                }
            }
        }
    }


    public function hookDisplayProductPriceBlock($param)
    {
        if (!$this->active) {
            return;
        }

        $activecheck=Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE');
        if($activecheck==0){
            return;
        }

        $product = $param['product'];
        $activemain = Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE');

        if (is_object($product)) {
            $catas = Product::getProductCategoriesFull($product->id);
            $productId = $product->id;
        } else {
            $catas = Product::getProductCategoriesFull($product['id_product']);
            $productId = $product['id_product'];
        }
        $customerGroups = array(Group::getCurrent()->id);


        $ruleData = AddifyrequestaquoteClass::getRuleByProduct($productId, $catas, $customerGroups);

    

        if ($ruleData) {

            $text = $ruleData['message'];
            $txtcolor = $ruleData['text_color'];
            $textbg = $ruleData['text_bg_color'];
            $hidePrice = $ruleData['hideprice'];
            $hideCart = $ruleData['hidecart'];

            if ($hidePrice==0) {
                $priceHideCheck = 'no';
            }
            else{
                $priceHideCheck = 'yes';
            }
            if ($hideCart==0) {
                $hideCartCheck = 'no';
            }
            else{
                $hideCartCheck = 'yes';
            }
            $hook_type = $param['type']; 
            if ( $hook_type == 'price') {
                if (isset($param['product'])) {
                    $this->context->smarty->assign(array(
                    'pattr' => $param['product']['id_product_attribute'],
                    ));
                }
            }
            
            $this->context->smarty->assign(array(
                'ps_version' => _PS_VERSION_,
                'hideCartCheck' => $hideCartCheck,
                'priceHideCheck'=>$priceHideCheck,
                'textbg'=>$textbg,
                'textcolor'=>$txtcolor,
                'text'=>$text,
                'productId'=>$productId,
                'activemain'=>$activemain,
            ));
            if ( $hook_type == 'price') {
                return $this->display(__FILE__, 'views/templates/hook/pattr.tpl').$this->display(__FILE__, 'views/templates/hook/extendhideprice.tpl');
            } else {
            return $this->display(__FILE__, 'views/templates/hook/extendhideprice.tpl');
            }
        } else {
            $this->context->smarty->assign(array(
                'ps_version' => _PS_VERSION_,
                'hideCartCheck' => 'no',
                'priceHideCheck'=> 'no',
                'textbg'=>'',
                'textcolor'=>'',
                'text'=>'',
                'productId'=>$productId,
                'activemain'=>$activemain,
            ));
            return $this->display(__FILE__, 'views/templates/hook/extendhideprice.tpl');
        }
    }

    public function hookDisplayGDPRConsent (){}
    public function hookRegisterGDPRConsent (){}

    public function hookDisplayTop($params)
    {
        $activecheck=Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE');
        if($activecheck==0){
            return;
        }
        $baseUrl = $this->context->shop->getBaseURL(true);
        $this->context->smarty->assign(array(
            'baseurl' => $baseUrl,
        ));
        return $this->display(__FILE__, 'views/templates/hook/addtoquoteicon.tpl');
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookDisplayBackOfficeHeader()
    {
        $link = new Link;
        $parameters = array("action" => "action_name");
        $addifyraq_ajax_link = $link->getAdminLink('AdminAddifyrequestaquoteCartlink&action=action_name', $parameters);

        Media::addJsDef(array(
            "addifyraq_ajax_link" => $addifyraq_ajax_link
        ));
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            $this->context->controller->addCSS($this->_path.'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $activecheck=Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE');
        if($activecheck==0){
            return;
        }
        $link = new Link;
        $parameters = array("action" => "action_name");
        $ajax_link = $link->getModuleLink('addifyrequestaquote','ajax', $parameters);

        Media::addJsDef(array(
            "ajax_raq_link" => $ajax_link
        ));
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');

        if(Tools::getValue('quotecart')=='true') {
            if(Tools::getValue('key')) {

                $haveKey = AddifyrequestsallClass::getKey(Tools::getValue('key'));

                if($haveKey){

                $getCartId = AddifycartsallClass::getCartId(Tools::getValue('key'));
                         
                if($getCartId[0]['cart_id']){
                    $this->context->cookie->__set('id_cart', $getCartId[0]['cart_id']);
                    Tools::redirect($this->context->shop->getBaseURL(true).'cart?action=show');
                }

                }
            }
        }
    }

    public function createCart($allpids,$allpattrids,$setallpprice,$pquantities){

        $allpids=explode(',',$allpids);
        $allpattrids=explode(',',$allpattrids);
        $setallpprice=explode(',',$setallpprice);
        $pquantities=explode(',',$pquantities);
       
        $new_cart = new Cart();
        $new_cart->id_currency = 1;
        $new_cart->id_lang = 1;
        $new_cart->save();

        for ($i=0; $i < count($allpids) ; $i++) { 
            $specific_price = new SpecificPrice();
            $specific_price->id_product = $allpids[$i]; // choosen product id-10
            $specific_price->id_product_attribute = $allpattrids[$i]; // optional or set to 0
            $specific_price->id_cart = $new_cart->id;
            $specific_price->from_quantity = $pquantities[$i];
            $specific_price->price = 150;
            $specific_price->reduction_type = 'percentage';
            $specific_price->reduction_tax = 1;
            $specific_price->id_shop =0;
            $specific_price->id_currency = 0;
            $specific_price->id_country = 0;
            $specific_price->id_group = 0;
            $specific_price->id_customer = 0;
            $specific_price->reduction = 0.0;
            $specific_price->from = date("Y-m-d H:i:s");
            $specific_price->to = "2020-10-10 06:06:06"; // or set date x days from now
            $specific_price->add();
            $new_cart->updateQty($pquantities[$i], $allpids[$i], $allpattrids[$i], false);
        }
    }

    public function sendMail($email,$name,$link) {

        Mail::send(
            $this->context->language->id,
            'sendcart',
            $this->l(
                'Requested Quote Cart Link'
            ),
            // Mail::l('Requested Quote Cart Link'),
            array(
                '{dateTime}' => date ('Y-m-d H:i:s'),
                '{link}' => $link
            ),
            $email,
            $name,
            Configuration::get('PS_SHOP_EMAIL'),
            Configuration::get('PS_SHOP_NAME'),
            null,
            null,
            dirname(__file__).'/mails/'

        );
    }

    public function hookDisplayCustomerAccount($params)
    { 
        $activecheck=Configuration::get('ADDIFYREQUESTAQUOTE_LIVE_MODE');
        if($activecheck==0){
            return;
        }
        $arq_tab_con_link = $this->context->link->getModuleLink($this->name,'quotehistorytab');
        $this->context->smarty->assign(array(
            'arq_tab_con_link' => $arq_tab_con_link,
        ));
        return $this -> display(__FILE__, '/views/templates/front/quote_history_tab.tpl');
    }

    public function hookActionDeleteGDPRCustomer ($customer){

        if (!empty($customer['id']) && Validate::isEmail($customer['email'])) {
            $sql = "DELETE FROM "._DB_PREFIX_."addifyrequestsall WHERE id_customer = '".pSQL($customer['id'])."'";
            if (Db::getInstance()->execute($sql)) {
                return json_encode(true);
            }
            return json_encode($this->l('Request a Quote Module : Unable to delete customer using email.'));
        }
    }

    public function hookActionExportGDPRData($customer){
        if (!Tools::isEmpty($customer['id']) && Validate::isEmail($customer['email'])) {
            $sql = "SELECT id_product AS `ID Product`,email AS `Email`,offered_price AS `Price Offered`,name AS `Name`,quantity AS `Quantity`,description AS `Description` FROM "._DB_PREFIX_."addifyrequestsall WHERE id_customer = '".pSQL($customer['id'])."'";
            if ($res = Db::getInstance()->ExecuteS($sql)) {



                return json_encode($res);
            } 
            return json_encode($this->l('Request a Quote Module : Unable to export customer using email.'));
        }
    }
}
